from openi.utils import file_utils, logger
from openi.settings import *
import os
import logging

logger.setup_logger()


"""
Dataset File class
"""


class DatasetFile:
    """
    File object in a dataset from https://openi.pcl.ac.cn/

    Attributes:
        filepath (`str`):
            filepath on local machine
        owner (`str`):
            username of target repo owner
        repository (`str`):
            repository name to which the dataset file is uploaded
        upload_type (`str`, [`0`,`1`], defaults to `1`):
            cloud storage type, `0` for GPU and `1` for NPU

        filename (`str`):
            filepath on local machine
        size (`str`):
            filepath on local machine
        upload_id (`str`):
            filepath on local machine
        uuid (`str`):
            filepath on local machine
        token (`str`):
            filepath on local machine
    """

    def __init__(self, file: str, repo_id: str, cluster: str, **kwargs):
        self.filepath = os.path.abspath(file)
        self.filename = os.path.basename(file)
        self.size = os.path.getsize(file)

        self.owner, self.repository = get_owner_repo(repo_id)
        self.upload_type, self.cluster = get_upload_type(cluster)

        self.chunk_size = (
            DATASET.SMALL_FILE_CHUNK_SIZE
            if self.size < DATASET.SMALL_FILE_LIMIT
            else DATASET.LARGE_FILE_CHUNK_SIZE
        )
        self.total_chunks_count = file_utils.get_file_chunk(
            chunk_size=self.chunk_size, filesize=self.size
        )

        self.uuid = None
        self.md5 = None
        self.upload_id = None
        self.dataset_id = None

    def upload_size_check(self):
        if self.size == 0:
            msg = f"❌ `{self.filename}` file size is 0"
            logging.error(msg)
            raise ValueError(msg)
        if self.size > DATASET.MAX_FILE_SIZE:
            msg = f"❌ `{self.filename}` file size exceeds {DATASET.MAX_FILE_SIZE_GB}GB"
            logging.error(msg)
            raise ValueError(msg)


def get_owner_repo(repo_id: str):
    try:
        owner = repo_id.split("/")[0]
        repository = repo_id.split("/")[1]
        return owner, repository
    except:
        msg = f"❌ argument `repo_id` must be in the form of `owner/repo`"
        logging.error(msg)
        raise ValueError(msg)


def get_upload_type(cluster: str):
    try:
        upload_type = DATASET.SOTRAGE_TYPE[cluster]
        cluster = cluster
        return upload_type, cluster
    except:
        msg = f"❌ argument `cluster` must be either `GPU` or `NPU`"
        logging.error(msg)
        raise ValueError(msg)



from openi.apis import OpeniAPI
from .dataset_file import *
from openi.utils import file_utils, logger
from openi.settings import *
from pathlib import Path
from tqdm import tqdm
import requests
import os
import logging

logger.setup_logger()


def download_with_tqdm(response: requests.Response, save_path: str, file: str):
    """
    Download file with tqdm progress bar
    """
    file_size = int(response.headers.get("content-length", 0))
    block_size = 1024  # 1 KB

    if not os.path.exists(save_path):
        os.makedirs(save_path)
    outfile = os.path.join(save_path, file)
    if os.path.exists(outfile):
        outfile = file_utils.rename_existing_file(outfile)

    with tqdm(
        total=file_size,
        leave=True,
        unit="B",
        unit_scale=True,
        unit_divisor=1000,
        bar_format="{desc}{percentage:3.0f}%|{bar}{r_bar}",
        desc=f"{logger.get_time()} - Downloading: ",
        dynamic_ncols=True,
    ) as pbar:
        with open(outfile, "wb") as f:
            for data in response.iter_content(block_size):
                pbar.update(len(data))
                f.write(data)
    print(f"{logger.get_time()} - 🎉 Download complete! file was saved to `{outfile}`")


def download_file(
    file: str,
    repo_id: str,
    cluster: str = "NPU",
    save_path: str = PATH.SAVE_PATH,
    endpoint: str = None,
    token: str = None,
):
    """
    Download file from dataset
    """

    _, repo = get_owner_repo(repo_id)
    upload_type, cluster = get_upload_type(cluster.upper())

    print(f"{logger.get_time()} - `{file}`({cluster}) preprocessing...")
    outfile = os.path.join(save_path, file)

    api = OpeniAPI(endpoint=endpoint, token=token)

    _datasets = api.get_dataset_attachment(repo_id=repo_id, upload_type=upload_type)
    if _datasets["data"] is not None:
        _dataset_info = next(
            (d for d in _datasets["data"] if d["repo"]["name"] == repo), None
        )
    if _datasets["count"] == 0 or _dataset_info is None:
        raise ValueError(
            f"❌ `{repo_id}`: dataset not created OR no zipfile({cluster.upper()}) found."
        )
    if _dataset_info["attachments"] is not None:
        _attachment = next(
            (d for d in _dataset_info["attachments"] if d["name"] == file), None
        )
    if _attachment is None:
        raise ValueError(
            f"❌ `{file}`({cluster.upper()}): file not found in `{repo_id}`"
        )
    uuid = _attachment["uuid"]

    response = api.download_attachments(uuid, upload_type)
    download_with_tqdm(response, save_path, file)
